/* Based on pyrandr (http://linux.mikeasoft.com/pyrandr.php) written
 * by Michael Sheldon (webmaster at mikeasoft dot com).
 */
#include <Python.h>
#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include <X11/extensions/Xrandr.h>

static PyObject *PyrandrError;

SizeID curSize;
XRRScreenConfiguration *screenConf;
XRRScreenSize *sizes;
Rotation rots, curRot;
Display *dpy;
Window root;

int available = 0;
short *rates;
int numRes=0, numRate=0;

static PyObject *pyrandr_randrAvailable(PyObject *self, PyObject *args) {
	return Py_BuildValue("b", available);
}

static PyObject *pyrandr_getCurRes(PyObject *self, PyObject *args) {
	if (available)
		return Py_BuildValue("i", curSize);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *pyrandr_getNumRes(PyObject *self, PyObject *args) {
	if (available)
		return Py_BuildValue("i", numRes);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *pyrandr_getRes(PyObject *self, PyObject *args) {
	int res;

	if(!PyArg_ParseTuple(args, "i", &res))
		return 0;

	if (available)
		return Py_BuildValue("(ii)", sizes[res].width, sizes[res].height);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *pyrandr_getCurRot(PyObject *self, PyObject *args) {
	if (available)
		return Py_BuildValue("i", curRot);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *pyrandr_getNumRot(PyObject *self, PyObject *args) {
	int rot;

	if(!PyArg_ParseTuple(args, "i", &rot))
		return 0;
	
	if (available)
		return Py_BuildValue("i", 0);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *pyrandr_getCurRate(PyObject *self, PyObject *args) {
	if (available)
		return Py_BuildValue("i", curRot - 1);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *pyrandr_getNumRates(PyObject *self, PyObject *args) {
	int res;
	
	if(!PyArg_ParseTuple(args, "i", &res))
		return 0;

	if (available) {
		rates = XRRConfigRates (screenConf, res, &numRate);
		return Py_BuildValue("i", numRate);
	}
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *pyrandr_getRate(PyObject *self, PyObject *args) {
	int rate, res;

	if(!PyArg_ParseTuple(args, "i", &res, &rate))
		return 0;

	if (available) {
		rates = XRRConfigRates (screenConf, res, &numRate);
		return Py_BuildValue("i", rates[rate]);
	}
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *pyrandr_setRes(PyObject *self, PyObject *args) {
	int setResolution;
	
	if(!PyArg_ParseTuple(args, "i", &setResolution))
		return 0;
	
	if (available) {
		Status status = RRSetConfigFailed;
		status = XRRSetScreenConfig (dpy, screenConf, root, setResolution, curRot, CurrentTime);
		screenConf = XRRGetScreenInfo (dpy, root);
		curSize = XRRConfigCurrentConfiguration (screenConf, &curRot);
		rates = XRRConfigRates (screenConf, curSize, &numRate);

		Py_INCREF(Py_None);
		return Py_None;
	}
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

//Method table
static PyMethodDef RandRMethods[] = {
	{"randrAvailable", pyrandr_randrAvailable, METH_VARARGS, "Checks to see if the RandR extension is available"},
	{"getCurRes", pyrandr_getCurRes, METH_VARARGS, "Return the current resolution's number"},
	{"getNumRes", pyrandr_getNumRes, METH_VARARGS, "Return the number of resolutions"},
	{"getRes", pyrandr_getRes, METH_VARARGS, "Return a resolution corresponding to a number"},
	{"getCurRot", pyrandr_getCurRot, METH_VARARGS, "Return the current rotation's number"},
	{"getNumRot", pyrandr_getNumRot, METH_VARARGS, "Return available rotations"},
	{"getNumRates", pyrandr_getNumRates, METH_VARARGS, "Return the number of refresh rates available at the specified resolution"},
	{"getCurRate", pyrandr_getCurRate, METH_VARARGS, "Return the current refresh rate"},
	{"getRate", pyrandr_getRate, METH_VARARGS, "Return the rate corresponding to a number"},
	{"setRes", pyrandr_setRes, METH_VARARGS, "Set the screen resolution"},
	{NULL, NULL, 0, NULL}
};
 
PyMODINIT_FUNC init_pyrandr(void) {
        PyObject *module = Py_InitModule("_pyrandr", RandRMethods);

	int event, err, screen = -1;

	PyrandrError = PyErr_NewException("_pyrandr.RandrNotAvailable", NULL, NULL);
	Py_INCREF(PyrandrError);
	PyModule_AddObject (module, "RandrNotAvailable", PyrandrError);

	if ((dpy = XOpenDisplay(NULL)) == NULL)
		return;

	/* Check we have RandR */
	available = XRRQueryExtension(dpy, &event, &err);
	if (available) {
		screen = DefaultScreen(dpy);
		root = RootWindow(dpy, screen);

		screenConf = XRRGetScreenInfo (dpy, root);
		curSize = XRRConfigCurrentConfiguration (screenConf, &curRot);
		sizes = XRRConfigSizes(screenConf, &numRes);
		rots = XRRConfigRotations(screenConf, &curRot);
	}
}
